/*
 * ARM JTAG Fault Injector
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef EMBICE_H
#define EMBICE_H

#include <stdint.h>

#define EMBICE_READ            0b000000
#define EMBICE_WRITE           0b100000

#define EMBICE_DEBUG_CTRL      0b00000
#define EMBICE_DEBUG_STATUS    0b00001
#define EMBICE_VECTOR_CATCH    0b00010
#define EMBICE_DEBUG_COMM_CTRL 0b00100
#define EMBICE_DEBUG_COMM_DATA 0b00101
#define EMBICE_WP0_ADDR        0b01000
#define EMBICE_WP0_ADDR_MASK   0b01001
#define EMBICE_WP0_DATA        0b01010
#define EMBICE_WP0_DATA_MASK   0b01011
#define EMBICE_WP0_CTRL        0b01100
#define EMBICE_WP0_CTRL_MASK   0b01101
#define EMBICE_WP1_ADDR        0b10000
#define EMBICE_WP1_ADDR_MASK   0b10001
#define EMBICE_WP1_DATA        0b10010
#define EMBICE_WP1_DATA_MASK   0b10011
#define EMBICE_WP1_CTRL        0b10100
#define EMBICE_WP1_CTRL_MASK   0b10101

/* DEBUG_CONTROL */
#define EMBICE_DBGACK  (1<<0)
#define EMBICE_DBGRQ   (1<<1)
#define EMBICE_INTDIS  (1<<2)
// bit 3: must be zero
#define EMBICE_MONITOR (1<<4)
#define EMBICE_DISABLE (1<<5)

/* DEBUG_STATUS */
// DBGACK same as in _CONTROL
// DBGRQ  same as in _CONTROL
#define EMBICE_IFEN    (1<<2)
#define EMBICE_SYSCOMP (1<<3)
#define EMBICE_ITBIT   (1<<4)
#define EMBICE_IJBIT   (1<<5)
// MOE not supported in Feroceon CPU

/* WPx_CTRL */
#define EMBICE_WPCTRL_ENABLE (1<<8)

void     embice_write_register(uint8_t reg, uint32_t v);
uint32_t embice_read_register(uint8_t reg);
void     embice_clear_breaks(void);

#endif
