/*
 * ARM JTAG Fault Injector
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef HWCONFIG_H
#define HWCONFIG_H

#include "autoconf.h"

#include <arm/NXP/LPC17xx/LPC17xx.h>
#include <arm/bits.h>

/* move performance-critical functions into RAM */
#define FAST_FUNCTIONS_IN_RAM

/* PLL settings for 100MHz from a 12MHz crystal */
#define PLL_MULTIPLIER 25
#define PLL_PREDIV     2
#define PLL_DIVISOR    3

/* SSP clock divisor (must be even) - 16.7MHz */
#define SSP_CLK_DIVISOR 6

/* don't change - periphal units used by the code */
#define SPI_ON_SSP 1
#define UART_NUMBER 0

/* delay between JTAG line toggles */
#define JTAG_DELAY() asm volatile("nop\nnop\nnop\nnop\nnop\n")
//nop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\nnop\n")
//#define JTAG_DELAY() delay_us(1)

#define JTAG_BIT_TMS  18    // DIP11
#define JTAG_BIT_TRST  0    // DIP9
#define JTAG_BIT_SRST  1    // DIP10
#define JTAG_BIT_TCK   7    // DIP7
#define JTAG_BIT_TDO   9    // DIP5
#define JTAG_BIT_TDI   8    // DIP6

#define TOUCHSCREEN_UART     2
#define TOUCHSCREEN_INT_GPIO LPC_GPIO2
#define TOUCHSCREEN_INT_PIN  5
#define TOUCHSCREEN_BAUD     9600

static inline void tsuart_connect(void) {
  LPC_PINCON->PINSEL0 |= BV(22);
}

/* note external connections:
   - GPIO0.7 to GPIO0.15 (DIP7 to DIP13) for SCK
   - GPIO0.6 to GPIO0.16 (DIP8 to DIP14) for SSEL
*/

/* inline functions for setting/resetting JTAG pins */
// FIXME: hardcoded for SSP1 pins
static inline __attribute__((always_inline)) void set_tms(int state) {
  if (state)
    LPC_GPIO0->FIOSET = BV(JTAG_BIT_TMS);
  else
    LPC_GPIO0->FIOCLR = BV(JTAG_BIT_TMS);
}

static inline __attribute__((always_inline)) void set_trst(int state) {
  if (state)
    LPC_GPIO0->FIOSET = BV(JTAG_BIT_TRST);
  else
    LPC_GPIO0->FIOCLR = BV(JTAG_BIT_TRST);
}

static inline __attribute__((always_inline)) void set_srst(int state) {
  if (state)
    LPC_GPIO0->FIOSET = BV(JTAG_BIT_SRST);
  else
    LPC_GPIO0->FIOCLR = BV(JTAG_BIT_SRST);
}

static inline __attribute__((always_inline)) void set_tck(int state) {
  if (state)
    LPC_GPIO0->FIOSET = BV(JTAG_BIT_TCK);
  else
    LPC_GPIO0->FIOCLR = BV(JTAG_BIT_TCK);
}

static inline __attribute__((always_inline)) void set_tdo(int state) {
  if (state)
    LPC_GPIO0->FIOSET = BV(JTAG_BIT_TDO);
  else
    LPC_GPIO0->FIOCLR = BV(JTAG_BIT_TDO);
}

static inline unsigned int get_tdi(void) {
  return !!(LPC_GPIO0->FIOPIN & BV(JTAG_BIT_TDI));
}

/* --- LEDs --- */

static inline __attribute__((always_inline)) void set_led0(int state) {
  if (state)
    LPC_GPIO1->FIOSET = BV(18);
  else
    LPC_GPIO1->FIOCLR = BV(18);
}

static inline __attribute__((always_inline)) void set_led1(int state) {
  if (state)
    LPC_GPIO1->FIOSET = BV(20);
  else
    LPC_GPIO1->FIOCLR = BV(20);
}

static inline __attribute__((always_inline)) void set_led2(int state) {
  if (state)
    LPC_GPIO1->FIOSET = BV(21);
  else
    LPC_GPIO1->FIOCLR = BV(21);
}

static inline __attribute__((always_inline)) void set_led3(int state) {
  if (state)
    LPC_GPIO1->FIOSET = BV(23);
  else
    LPC_GPIO1->FIOCLR = BV(23);
}


#endif
