/*
 * ARM JTAG Fault Injector
 *
 * Author: Andreas Heinig <andreas.heinig@gmx.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INJTYPES_H__INCLUDED__
#define INJTYPES_H__INCLUDED__

#include <stdint.h>

#define ERROR_TYPE_UNKNOWN	0
#define ERROR_TYPE_MEMORY	1
#define ERROR_TYPE_REGISTER	2

typedef struct {
	uint32_t start;
	uint32_t size;
} mem_range_t;

typedef struct {
	volatile unsigned int ival_inst_check_cnt;
	volatile unsigned int inst_load_cnt;
	volatile unsigned int inst_load_noecc_cnt;
	volatile unsigned int inst_store_cnt;
	volatile unsigned int inst_store_noecc_cnt;
	volatile unsigned int mem_inject_cnt;
	volatile unsigned int reg_inject_cnt;
	volatile unsigned int sys_cnt;
} inj_stat_t;

#define ENABLE_INJECT		0x0001
#define INJECT_TO_MEM		0x0002
#define INJECT_TO_REG		0x0004
#define ENABLE_FLIP		0x0008
#define RECORD_MEM		0x0010
#define SINGLE_STEP_MODE	0x0020
//#define USE_MEM_MAP		0x0040
#define ADDRESS_ONLY		0x0080
#define DISABLE_AWARENESS	0x0100
#define INJECT_ONLY_SYS_MODES	0x0200
#define OPTION_Y		0x40000000
#define OPTION_Z		0x80000000
#endif /* INJTYPES_H__INCLUDED__ */
