/*
 * ARM JTAG Fault Injector
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef JTAG_H
#define JTAG_H

/* JTAG states */
typedef enum {
  JTST_UNKNOWN, JTST_RESET, JTST_IDLE,
  JTST_SELECT_DR, JTST_CAPTURE_DR, JTST_SHIFT_DR,
  JTST_EXIT1_DR, JTST_PAUSE_DR, JTST_EXIT2_DR, JTST_UPDATE_DR,
  JTST_SELECT_IR, JTST_CAPTURE_IR, JTST_SHIFT_IR,
  JTST_EXIT1_IR, JTST_PAUSE_IR, JTST_EXIT2_IR, JTST_UPDATE_IR
} jtag_state_t;

/* ARM-JTAG specific instruction register codes */
#define JTAG_IR_LEN 4
#define JTAG_EXTEXT        0b0000
#define JTAG_SAMPLEPRELOAD 0b0011
#define JTAG_SCAN_N        0b0010
#define JTAG_INTEST        0b1100
#define JTAG_IDCODE        0b1110
#define JTAG_BYPASS        0b1111
#define JTAG_RESTART       0b0100

#define CHAIN_CORE     1
#define CHAIN_ICE      2
#define CHAIN_BOUNDARY 3
/* unverified, from openocd */
#define CHAIN_ETM      6
#define CHAIN_COP15    15

extern unsigned int jtag_clocks;

extern const unsigned char bit_reverse_table256[];

typedef struct {
  unsigned int bits;
  void        *output;
  void        *input;
} jtag_block_t;

void jtag_init(void);

void     jtag_idle(void);
uint32_t jtag_scan_ir(uint32_t value, unsigned int bits, jtag_state_t endstate);
uint32_t jtag_scan_dr_single(uint32_t value, unsigned int bits, jtag_state_t endstate);
void     jtag_select_chain(unsigned int ch, jtag_state_t endstate);
void     jtag_change_state(jtag_state_t target, unsigned int force);

/* same as jtag_scan_ir, but just returns if value is identical to the last IR value set */
void jtag_set_ir(uint32_t value, unsigned int bits, jtag_state_t endstate);

/* runs an array of jtag_block_t through DR */
void jtag_scan_dr(unsigned int count, jtag_block_t *blocks, jtag_state_t endstate);

#endif
