/*
 * ARM JTAG Fault Injector
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "hwconfig.h"
#include <stddef.h>
#include "jtag.h"
#include "embice.h"

uint32_t embice_read_register(uint8_t reg)
{
	uint32_t value_in;
	jtag_block_t bl[2] = {
			{ 32, NULL, &value_in },
			{ 6,  &reg, NULL }
	};

	jtag_select_chain(CHAIN_ICE, JTST_IDLE);
	jtag_set_ir(JTAG_INTEST, JTAG_IR_LEN, JTST_IDLE);

	jtag_scan_dr(2, bl, JTST_IDLE); // select
	jtag_scan_dr(2, bl, JTST_IDLE); // read - FIXME: change reg num to avoid reading DCC twice
	return value_in;
}

void embice_write_register(uint8_t reg, uint32_t v)
{
	jtag_block_t bl[2] = {
			{ 32, &v,  NULL },
			{ 6, &reg, NULL }
	};

	reg |= EMBICE_WRITE;

	jtag_select_chain(CHAIN_ICE, JTST_IDLE);
	jtag_set_ir(JTAG_INTEST, JTAG_IR_LEN, JTST_IDLE);

	jtag_scan_dr(2, bl, JTST_IDLE);
}

/* disable everything in EmbeddedICE that may halt the CPU */
void embice_clear_breaks(void)
{
	embice_write_register(EMBICE_VECTOR_CATCH, 0);
	embice_write_register(EMBICE_WP0_CTRL, 0);
	embice_write_register(EMBICE_WP1_CTRL, 0);
}
