/*
 * ARM JTAG Fault Injector
 * - startup code for LPC17xx
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "asmconfig.h"
        
        .syntax unified

        .section .vectors

        .macro  except label
        .weak   \label
        .set    \label, __unhandled_exception
        .word   \label
        .endm
        
        /* Cortex M3 standard except vectors */
        .word   __stack
        .word   _start
#ifndef CONFIG_VECTORHUNTER
        except  NMI_Handler
        except  HardFault_Handler  /* 2 */
        except  MemManage_Handler
        except  BusFault_Handler   /* 4 */
        except  UsageFault_Handler
        .word   0
        .word   0
        .word   0
        .word   0
        except  SVC_Handler     /* 10 */
        except  DebugMon_Handler
        .word   0
        except  PendSV_Handler
        except  SysTick_Handler

        /* External interrupt vectors */
        except  WDT_IRQHandler
        except  TIMER0_IRQHandler
        except  TIMER1_IRQHandler
        except  TIMER2_IRQHandler
        except  TIMER3_IRQHandler
        except  UART0_IRQHandler  /* 20 */
        except  UART1_IRQHandler
        except  UART2_IRQHandler
        except  UART3_IRQHandler
        except  PWM1_IRQHandler
        except  I2C0_IRQHandler
        except  I2C1_IRQHandler
        except  I2C2_IRQHandler  /* 27 */
        except  SPI_IRQHandler
        except  SSP0_IRQHandler
        except  SSP1_IRQHandler
        except  PLL0_IRQHandler
        except  RTC_IRQHandler
        except  EINT0_IRQHandler
        except  EINT1_IRQHandler
        except  EINT2_IRQHandler
        except  EINT3_IRQHandler
        except  ADC_IRQHandler
        except  BOD_IRQHandler
        except  USB_IRQHandler
        except  CAN_IRQHandler
        except  DMA_IRQHandler
        except  I2S_IRQHandler
        except  ENET_IRQHandler
        except  RIT_IRQHandler
        except  MCPWM_IRQHandler
        except  QEI_IRQHandler
        except  PLL1_IRQHandler
#else
        ;; Vectorhunter
        .irp    number,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
        .word   vec\number
        .endr
#endif

        .section .text

        .global _start
        .thumb_func
_start:
        /* copy data section to ram */
        ldr     r0, =__data_load_start
        ldr     r1, =__data_load_end
        ldr     r2, =__data_start
dataloop:
        ldr.w   r3, [r0], #4
        str.w   r3, [r2], #4
        cmp     r0, r1
        blo     dataloop

        /* clear bss section */
        ldr     r0, =__bss_start__
        ldr     r1, =__bss_end__
        ldr     r2, =0
bssloop:
        str.w   r2, [r0], #4
        cmp     r0, r1
        blo     bssloop

        /* start main() */
        b       main

        
        /* dummy exception handler: endless loop */
        .weak __unhandled_exception
        .thumb_func
__unhandled_exception:
        ldr     r0, =0x2009c038
        ldr     r1, =0x800000
        str     r1, [r0]
        b       __unhandled_exception

#ifdef CONFIG_VECTORHUNTER
        .global vectorhunter
        .irp    number,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
        .thumb_func
vec\number :
        ldr     r0, =\number
        b       vectorhunter
        .endr
#endif

        
        .end
