/*
 * ARM JTAG Fault Injector
 *
 * Author: Ingo Korb <ingo@akana.de>
 *
 * Copyright (C) 2011-2014 Department of Computer Science,
 * Design Automation of Embedded Systems Group,
 * Dortmund University of Technology, all rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   3. Neither the name of the copyright holder nor the names of its
 *      contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <arm/NXP/LPC17xx/LPC17xx.h>
#include <arm/bits.h>
#include "hwconfig.h"
#include "timer.h"
#include "parameters.h"

/* bit definitions */
#define RITINT 0
#define RITEN  3

#define PCRIT 16

volatile tick_t ticks;

void __attribute__((weak,noinline)) SysTick_Hook(void)
				{
	// Empty function for hooking the systick handler
				}

/* Systick interrupt handler */
void SysTick_Handler(void)
{
	ticks++;
	SysTick_Hook();
}

void timer_init(void)
{
	/* turn on power to RIT */
	BITBAND(LPC_SC->PCONP, PCRIT) = 1;

	/* clear RIT mask */
	LPC_RIT->RIMASK = 0; //xffffffff;

	/* PCLK = CCLK */
	BITBAND(LPC_SC->PCLKSEL1, 26) = 1;

	/* enable SysTick */
	//SysTick_Config(SysTick->CALIB & SysTick_CALIB_TENMS_Msk);
	SysTick_Config((CONFIG_MCU_FREQ / 1000) * PARAM_TICK_INTERVAL);
}

void delay_us(unsigned int time)
{
	/* Prepare RIT */
	LPC_RIT->RICOUNTER = 0;
	LPC_RIT->RICOMPVAL = (CONFIG_MCU_FREQ / 1000000) * time;
	LPC_RIT->RICTRL    = BV(RITEN) | BV(RITINT);

	/* Wait until RIT signals an interrupt */
	while (!(BITBAND(LPC_RIT->RICTRL, RITINT))) ;

	/* Disable RIT */
	LPC_RIT->RICTRL = 0;
}

void delay_ms(unsigned int time)
{
	/* Prepare RIT */
	LPC_RIT->RICOUNTER = 0;
	LPC_RIT->RICTRL    = BV(RITEN) | BV(RITINT);
	LPC_RIT->RICOMPVAL = (CONFIG_MCU_FREQ / 1000) * time;

	/* Wait until RIT signals an interrupt */
	while (!(BITBAND(LPC_RIT->RICTRL, RITINT))) ;

	/* Disable RIT */
	LPC_RIT->RICTRL = 0;
}

